$VAR1 = [

	{   'DepartmentType' => [
            [ 'department_type_id', 'name', ],
            [ 1,      'Sale' ],
            [ 2,      'Bioinformatics' ],
            [ 3,      'Marketing' ]
        ],
    },

];